/***********************************************************************/
/*                                                                     */
/* Program: koch.c                                                     */
/*                                                                     */
/* The algorithm is taken from the book:                               */
/*                                                                     */
/* "Fractals for the Classroom" part one, Springer Verlag by           */
/* H-O. Peitgen, H. Juurgens and D. Saupe.                             */
/*                                                                     */
/* Parameters:                                                         */
/*                                                                     */
/* image: Output image name.                                           */
/* size : Image size in pixels (512).                                  */
/* col  : Greyscale colour (0 <= col < 32768) (0).                     */
/* lev  : Recursion level (4).                                         */
/*                                                                     */
/*                                                                     */
/* Program by:    Stefan Spaennare, Lund, Sweden                       */
/* E-mail:        stefan@spaennare.se                                  */
/*                                                                     */
/* First version: June 1995                                            */
/* Latest update: 2004-06-23                                           */
/*                                                                     */
/***********************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <time.h>
#include "image.h"

double timetic=(double)(CLOCKS_PER_SEC);

double pi=3.14159265358979323;

void line(im,x1,y1,x2,y2,col)
short **im;
int x1,y1,x2,y2;
int col;
{
   double dx,dy,r,rr,ax,ay,x,y;
   int i,xi,yi,ri;

   dx=(double)(x2)-(double)(x1);
   dy=(double)(y2)-(double)(y1);

   r=sqrt(dx*dx+dy*dy);

   ri=2*(int)(r+1.0);

   rr=(double)(ri);

   ax=dx/rr;
   ay=dy/rr;

   x=(double)(x1);
   y=(double)(y1);

   for (i=0; i<=ri; i++) {
      xi=(int)(x+0.5);
      yi=(int)(y+0.5);
      im[yi][xi]=col;
      x=x+ax;
      y=y+ay;
   } /* for i */

} /* line */


void recu(image,xl,xr,yl,yr,r,lev,col)
short **image;
double *xl,*xr,*yl,*yr;
double r;
int *lev;
int col;
{
   int xli,yli,xri,yri;

   double inv3;

   inv3=1.0/3.0;

   if (*lev == 1) {

      xli=(int)(xl[1]+0.5);
      yli=(int)(yl[1]+0.5);
      xri=(int)(xr[1]+0.5);
      yri=(int)(yr[1]+0.5);

      line(image,xli,yli,xri,yri,col);

      return;

   }
   else {
      (*lev)--;

      xl[*lev]=xl[*lev+1];
      yl[*lev]=yl[*lev+1];
      xr[*lev]=inv3*xr[*lev+1]+2.0*inv3*xl[*lev+1]; 
      yr[*lev]=inv3*yr[*lev+1]+2.0*inv3*yl[*lev+1]; 

      recu(image,xl,xr,yl,yr,r,lev,col);

      xl[*lev]=xr[*lev];
      yl[*lev]=yr[*lev];
      xr[*lev]=0.5*xr[*lev+1]+0.5*xl[*lev+1]-r*(yl[*lev+1]-yr[*lev+1]);
      yr[*lev]=0.5*yr[*lev+1]+0.5*yl[*lev+1]+r*(xl[*lev+1]-xr[*lev+1]);

      recu(image,xl,xr,yl,yr,r,lev,col);

      xl[*lev]=xr[*lev];
      yl[*lev]=yr[*lev];
      xr[*lev]=2.0*inv3*xr[*lev+1]+inv3*xl[*lev+1];
      yr[*lev]=2.0*inv3*yr[*lev+1]+inv3*yl[*lev+1];

      recu(image,xl,xr,yl,yr,r,lev,col);

      xl[*lev]=xr[*lev];
      yl[*lev]=yr[*lev];
      xr[*lev]=xr[*lev+1];
      yr[*lev]=yr[*lev+1];

      recu(image,xl,xr,yl,yr,r,lev,col);

      (*lev)++;

      return;

   } /* if */

} /* recu */




int main(argc,argv)
int argc;
char *argv[];
{
   struct image_head header;    
   short **image;

   double flops,ftime;

   double r,ss,yy;

   int lev,i,j,size,lev0,col;

   double *xl,*xr,*yl,*yr;

   clock_t time1,timediff;


   if (argc<4) {
      printf("Usage: %s image sizei coli levi\n",argv[0]);
      exit(0);
   } /* if */

   size=atoi(argv[2]);
   col=atoi(argv[3]);
   lev0=atoi(argv[4]);


   header.id=image_magic;
   header.typ=im_s16;
   header.off=512;
   header.hgh=size;
   header.wid=size;
 
   image = (short **)calloc(header.hgh,sizeof(short*));

   CreateIm_s16(image,header.hgh,header.wid);

   xl=(double *)calloc(lev0+1,sizeof(double));
   xr=(double *)calloc(lev0+1,sizeof(double));
   yl=(double *)calloc(lev0+1,sizeof(double));
   yr=(double *)calloc(lev0+1,sizeof(double));


for (i=0; i<size; i++) {
   for (j=0; j<size; j++) {
       image[i][j]=32767;
    } /* for j */
} /* for i */


time1=clock();


ss=(double)(size)/1024.0;
r=0.29;
lev=lev0;

yy=750.0-sqrt(800.0*800.0-400.0*400.0);

xl[lev]=500.0*ss;
yl[lev]=yy*ss;
xr[lev]=900.0*ss;
yr[lev]=750.0*ss;

recu(image,xl,xr,yl,yr,r,&lev,col);

r=0.29;
lev=lev0;

xl[lev]=900.0*ss;
yl[lev]=750.0*ss;
xr[lev]=100.0*ss;
yr[lev]=750.0*ss;

recu(image,xl,xr,yl,yr,r,&lev,col);

r=0.29;
lev=lev0;

xl[lev]=100.0*ss;
yl[lev]=750.0*ss;
xr[lev]=500.0*ss;
yr[lev]=yy*ss;

recu(image,xl,xr,yl,yr,r,&lev,col);


timediff=clock()-time1;

ftime=(double)(timediff)/timetic;

printf("\n");
printf("Time:  %12.2f\n",ftime);

WriteImage(argv[1],image,header,im_s16);

} /* End */
